# coding: utf-8

# (C) Copyright IBM Corp. 2023.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# IBM OpenAPI SDK Code Generator Version: 3.70.0-7df966bf-20230419-195904

"""
Manage the lifecycle of your users using User Management APIs.

API Version: 1.0
"""

from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse, get_query_param
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import convert_model

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################


class UserManagementV1(BaseService):
    """The User Management V1 service."""

    DEFAULT_SERVICE_URL = 'https://user-management.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'user_management'

    @classmethod
    def new_instance(
        cls,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> 'UserManagementV1':
        """
        Return a new client for the User Management service using the specified
               parameters and external configuration.
        """
        authenticator = get_authenticator_from_environment(service_name)
        service = cls(authenticator)
        service.configure_service(service_name)
        return service

    def __init__(
        self,
        authenticator: Authenticator = None,
    ) -> None:
        """
        Construct a new client for the User Management service.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/main/README.md
               about initializing the authenticator of your choice.
        """
        BaseService.__init__(self, service_url=self.DEFAULT_SERVICE_URL, authenticator=authenticator)

    #########################
    # Users
    #########################

    def list_users(
        self,
        account_id: str,
        *,
        limit: int = None,
        include_settings: bool = None,
        search: str = None,
        start: str = None,
        user_id: str = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        List users.

        Retrieve users in the account. You can use the IAM service token or a user token
        for authorization. To use this method, the requesting user or service ID must have
        at least the viewer, editor, or administrator role on the User Management service.
        If unrestricted view is enabled, the user can see all users in the same account
        without an IAM role. If restricted view is enabled and user has the viewer,
        editor, or administrator role on the user management service, the API returns all
        users in the account. If unrestricted view is enabled and the user does not have
        these roles, the API returns only the current user. Users are returned in a
        paginated list with a default limit of 100 users. You can iterate through all
        users by following the `next_url` field. Additional substring search fields are
        supported to filter the users.

        :param str account_id: The account ID of the specified user.
        :param int limit: (optional) The number of results to be returned.
        :param bool include_settings: (optional) The user settings to be returned.
               Set to true to view language, allowed IP address, and authentication
               settings.
        :param str search: (optional) The desired search results to be returned. To
               view the list of users with the additional search filter, use the following
               query options: `firstname`, `lastname`, `email`, `state`, `substate`,
               `iam_id`, `realm`, and `userId`. HTML URL encoding for the search query and
               `:` must be used. For example, search=state%3AINVALID returns a list of
               invalid users. Multiple search queries can be combined to obtain `OR`
               results using `,` operator (not URL encoded). For example,
               search=state%3AINVALID,email%3Amail.test.ibm.com.
        :param str start: (optional) An optional token that indicates the beginning
               of the page of results to be returned. If omitted, the first page of
               results is returned. This value is obtained from the 'next_url' field of
               the operation response.
        :param str user_id: (optional) Filter users based on their user ID.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `UserList` object
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='list_users',
        )
        headers.update(sdk_headers)

        params = {
            'limit': limit,
            'include_settings': include_settings,
            'search': search,
            '_start': start,
            'user_id': user_id,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_id']
        path_param_values = self.encode_path_vars(account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/accounts/{account_id}/users'.format(**path_param_dict)
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response

    def invite_users(
        self,
        account_id: str,
        *,
        users: List['InviteUser'] = None,
        iam_policy: List['InviteUserIamPolicy'] = None,
        access_groups: List[str] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Invite users to an account.

        Invite users to the account. You must use a user token for authorization. Service
        IDs can't invite users to the account. To use this method, the requesting user
        must have the editor or administrator role on the User Management service. For
        more information, see the [Inviting
        users](https://cloud.ibm.com/docs/account?topic=account-iamuserinv) documentation.
        You can specify the user account role and the corresponding IAM policy information
        in the request body. <br/><br/>When you invite a user to an account, the user is
        initially created in the `PROCESSING` state. After the user is successfully
        created, all specified permissions are configured, and the activation email is
        sent, the invited user is transitioned to the `PENDING` state. When the invited
        user clicks the activation email and creates and confirms their IBM Cloud account,
        the user is transitioned to `ACTIVE` state. If the user email is already verified,
        no email is generated.

        :param str account_id: The account ID of the specified user.
        :param List[InviteUser] users: (optional) A list of users to be invited.
        :param List[InviteUserIamPolicy] iam_policy: (optional) A list of IAM
               policies.
        :param List[str] access_groups: (optional) A list of access groups.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `InvitedUserList` object
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if users is not None:
            users = [convert_model(x) for x in users]
        if iam_policy is not None:
            iam_policy = [convert_model(x) for x in iam_policy]
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='invite_users',
        )
        headers.update(sdk_headers)

        data = {
            'users': users,
            'iam_policy': iam_policy,
            'access_groups': access_groups,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_id']
        path_param_values = self.encode_path_vars(account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/accounts/{account_id}/users'.format(**path_param_dict)
        request = self.prepare_request(
            method='POST',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def get_user_profile(
        self,
        account_id: str,
        iam_id: str,
        *,
        include_activity: str = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get user profile.

        Retrieve a user's profile by the user's IAM ID in your account. You can use the
        IAM service token or a user token for authorization. To use this method, the
        requesting user or service ID must have at least the viewer, editor, or
        administrator role on the User Management service.

        :param str account_id: The account ID of the specified user.
        :param str iam_id: The user's IAM ID.
        :param str include_activity: (optional) Include activity information of the
               user, such as the last authentication timestamp.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `UserProfile` object
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if not iam_id:
            raise ValueError('iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_user_profile',
        )
        headers.update(sdk_headers)

        params = {
            'include_activity': include_activity,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_id', 'iam_id']
        path_param_values = self.encode_path_vars(account_id, iam_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/accounts/{account_id}/users/{iam_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response

    def update_user_profile(
        self,
        account_id: str,
        iam_id: str,
        *,
        firstname: str = None,
        lastname: str = None,
        state: str = None,
        email: str = None,
        phonenumber: str = None,
        altphonenumber: str = None,
        photo: str = None,
        include_activity: str = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Partially update user profile.

        Partially update a user's profile by user's IAM ID. You can use the IAM service
        token or a user token for authorization. To use this method, the requesting user
        or service ID must have at least the editor or administrator role on the User
        Management service. A user or service ID with these roles can change a user's
        state between `ACTIVE`, `VPN_ONLY`, or `DISABLED_CLASSIC_INFRASTRUCTURE`, but they
        can't change the state to `PROCESSING` or `PENDING` because these are system
        states. For other request body fields, a user can update their own profile without
        having User Management service permissions.

        :param str account_id: The account ID of the specified user.
        :param str iam_id: The user's IAM ID.
        :param str firstname: (optional) The first name of the user.
        :param str lastname: (optional) The last name of the user.
        :param str state: (optional) The state of the user. Possible values are
               `PROCESSING`, `PENDING`, `ACTIVE`, `DISABLED_CLASSIC_INFRASTRUCTURE`, and
               `VPN_ONLY`.
        :param str email: (optional) The email address of the user.
        :param str phonenumber: (optional) The phone number of the user.
        :param str altphonenumber: (optional) The alternative phone number of the
               user.
        :param str photo: (optional) A link to a photo of the user.
        :param str include_activity: (optional) Include activity information of the
               user, such as the last authentication timestamp.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if not iam_id:
            raise ValueError('iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='update_user_profile',
        )
        headers.update(sdk_headers)

        params = {
            'include_activity': include_activity,
        }

        data = {
            'firstname': firstname,
            'lastname': lastname,
            'state': state,
            'email': email,
            'phonenumber': phonenumber,
            'altphonenumber': altphonenumber,
            'photo': photo,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_id', 'iam_id']
        path_param_values = self.encode_path_vars(account_id, iam_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/accounts/{account_id}/users/{iam_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='PATCH',
            url=url,
            headers=headers,
            params=params,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def remove_user(
        self,
        account_id: str,
        iam_id: str,
        *,
        include_activity: str = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Remove user from account.

        Remove users from an account by user's IAM ID. You must use a user token for
        authorization. Service IDs can't remove users from an account. To use this method,
        the requesting user must have the editor or administrator role on the User
        Management service. For more information, see the [Removing
        users](https://cloud.ibm.com/docs/account?topic=account-remove) documentation.

        :param str account_id: The account ID of the specified user.
        :param str iam_id: The user's IAM ID.
        :param str include_activity: (optional) Include activity information of the
               user, such as the last authentication timestamp.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if not iam_id:
            raise ValueError('iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='remove_user',
        )
        headers.update(sdk_headers)

        params = {
            'include_activity': include_activity,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_id', 'iam_id']
        path_param_values = self.encode_path_vars(account_id, iam_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/accounts/{account_id}/users/{iam_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='DELETE',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response

    def accept(
        self,
        *,
        account_id: str = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Accept an invitation.

        Accept a user invitation to an account. You can use the user's token for
        authorization. To use this method, the requesting user must provide the account ID
        for the account that they are accepting an invitation for. If the user already
        accepted the invitation request, it returns 204 with no response body.

        :param str account_id: (optional) The account ID.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='accept',
        )
        headers.update(sdk_headers)

        data = {
            'account_id': account_id,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        url = '/v2/users/accept'
        request = self.prepare_request(
            method='POST',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def v3_remove_user(
        self,
        account_id: str,
        iam_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Remove user from account (Asynchronous).

        Remove users from an account by using the user's IAM ID. You must use a user token
        for authorization. Service IDs can't remove users from an account. If removing the
        user fails it will set the user's state to ERROR_WHILE_DELETING. To use this
        method, the requesting user must have the editor or administrator role on the User
        Management service. For more information, see the [Removing
        users](https://cloud.ibm.com/docs/account?topic=account-remove) documentation.

        :param str account_id: The account ID of the specified user.
        :param str iam_id: The user's IAM ID.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if not iam_id:
            raise ValueError('iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='v3_remove_user',
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_id', 'iam_id']
        path_param_values = self.encode_path_vars(account_id, iam_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v3/accounts/{account_id}/users/{iam_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='DELETE',
            url=url,
            headers=headers,
        )

        response = self.send(request, **kwargs)
        return response

    #########################
    # User Settings
    #########################

    def get_user_settings(
        self,
        account_id: str,
        iam_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get user settings.

        Retrieve a user's settings by the user's IAM ID. You can use the IAM service token
        or a user token for authorization. To use this method, the requesting user or
        service ID must have the viewer, editor, or administrator role on the User
        Management service. <br/><br/>The user settings have several fields. The
        `language` field is the language setting for the user interface display language.
        The `notification_language` field is the language setting for phone and email
        notifications. The `allowed_ip_addresses` field specifies a list of IP addresses
        that the user can log in and perform operations from as described in [Allowing
        specific IP addresses for a
        user](https://cloud.ibm.com/docs/account?topic=account-ips). For information about
        the `self_manage` field, review information about the [user-managed login
        setting](https://cloud.ibm.com/docs/account?topic=account-types).

        :param str account_id: The account ID of the specified user.
        :param str iam_id: The user's IAM ID.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `UserSettings` object
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if not iam_id:
            raise ValueError('iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_user_settings',
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_id', 'iam_id']
        path_param_values = self.encode_path_vars(account_id, iam_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/accounts/{account_id}/users/{iam_id}/settings'.format(**path_param_dict)
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
        )

        response = self.send(request, **kwargs)
        return response

    def update_user_settings(
        self,
        account_id: str,
        iam_id: str,
        *,
        language: str = None,
        notification_language: str = None,
        allowed_ip_addresses: str = None,
        self_manage: bool = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Partially update user settings.

        Update a user's settings by the user's IAM ID. You can use the IAM service token
        or a user token for authorization. To fully use this method, the user or service
        ID must have the editor or administrator role on the User Management service.
        Without these roles, a user can update only their own `language` or
        `notification_language` fields. If `self_manage` is `true`, the user can also
        update the `allowed_ip_addresses` field.

        :param str account_id: The account ID of the specified user.
        :param str iam_id: The user's IAM ID.
        :param str language: (optional) The console UI language. By default, this
               field is empty.
        :param str notification_language: (optional) The language for email and
               phone notifications. By default, this field is empty.
        :param str allowed_ip_addresses: (optional) A comma-separated list of IP
               addresses.
        :param bool self_manage: (optional) Whether user managed login is enabled.
               The default value is `false`.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if not iam_id:
            raise ValueError('iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='update_user_settings',
        )
        headers.update(sdk_headers)

        data = {
            'language': language,
            'notification_language': notification_language,
            'allowed_ip_addresses': allowed_ip_addresses,
            'self_manage': self_manage,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_id', 'iam_id']
        path_param_values = self.encode_path_vars(account_id, iam_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/accounts/{account_id}/users/{iam_id}/settings'.format(**path_param_dict)
        request = self.prepare_request(
            method='PATCH',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response


##############################################################################
# Models
##############################################################################


class InvitedUser:
    """
    Information about a user that has been invited to join an account.

    :attr str email: (optional) The email address associated with the invited user.
    :attr str id: (optional) The id associated with the invited user.
    :attr str state: (optional) The state of the invitation for the user.
    """

    def __init__(
        self,
        *,
        email: str = None,
        id: str = None,
        state: str = None,
    ) -> None:
        """
        Initialize a InvitedUser object.

        :param str email: (optional) The email address associated with the invited
               user.
        :param str id: (optional) The id associated with the invited user.
        :param str state: (optional) The state of the invitation for the user.
        """
        self.email = email
        self.id = id
        self.state = state

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'InvitedUser':
        """Initialize a InvitedUser object from a json dictionary."""
        args = {}
        if 'email' in _dict:
            args['email'] = _dict.get('email')
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'state' in _dict:
            args['state'] = _dict.get('state')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a InvitedUser object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'email') and self.email is not None:
            _dict['email'] = self.email
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this InvitedUser object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'InvitedUser') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'InvitedUser') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class InvitedUserList:
    """
    A collection of invited users.  This is the response returned by the invite_users
    operation.

    :attr List[InvitedUser] resources: (optional) The list of users that have been
          invited to join the account.
    """

    def __init__(
        self,
        *,
        resources: List['InvitedUser'] = None,
    ) -> None:
        """
        Initialize a InvitedUserList object.

        :param List[InvitedUser] resources: (optional) The list of users that have
               been invited to join the account.
        """
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'InvitedUserList':
        """Initialize a InvitedUserList object from a json dictionary."""
        args = {}
        if 'resources' in _dict:
            args['resources'] = [InvitedUser.from_dict(v) for v in _dict.get('resources')]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a InvitedUserList object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this InvitedUserList object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'InvitedUserList') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'InvitedUserList') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class UserList:
    """
    The users returned.

    :attr int total_results: The number of users returned.
    :attr int limit: A limit to the number of users returned in a page.
    :attr str first_url: (optional) The first URL of the get users API.
    :attr str next_url: (optional) The next URL of the get users API.
    :attr List[UserProfile] resources: (optional) A list of users in the account.
    """

    def __init__(
        self,
        total_results: int,
        limit: int,
        *,
        first_url: str = None,
        next_url: str = None,
        resources: List['UserProfile'] = None,
    ) -> None:
        """
        Initialize a UserList object.

        :param int total_results: The number of users returned.
        :param int limit: A limit to the number of users returned in a page.
        :param str first_url: (optional) The first URL of the get users API.
        :param str next_url: (optional) The next URL of the get users API.
        :param List[UserProfile] resources: (optional) A list of users in the
               account.
        """
        self.total_results = total_results
        self.limit = limit
        self.first_url = first_url
        self.next_url = next_url
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'UserList':
        """Initialize a UserList object from a json dictionary."""
        args = {}
        if 'total_results' in _dict:
            args['total_results'] = _dict.get('total_results')
        else:
            raise ValueError('Required property \'total_results\' not present in UserList JSON')
        if 'limit' in _dict:
            args['limit'] = _dict.get('limit')
        else:
            raise ValueError('Required property \'limit\' not present in UserList JSON')
        if 'first_url' in _dict:
            args['first_url'] = _dict.get('first_url')
        if 'next_url' in _dict:
            args['next_url'] = _dict.get('next_url')
        if 'resources' in _dict:
            args['resources'] = [UserProfile.from_dict(v) for v in _dict.get('resources')]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a UserList object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'total_results') and self.total_results is not None:
            _dict['total_results'] = self.total_results
        if hasattr(self, 'limit') and self.limit is not None:
            _dict['limit'] = self.limit
        if hasattr(self, 'first_url') and self.first_url is not None:
            _dict['first_url'] = self.first_url
        if hasattr(self, 'next_url') and self.next_url is not None:
            _dict['next_url'] = self.next_url
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this UserList object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'UserList') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'UserList') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class UserProfile:
    """
    Returned the user profile.

    :attr str id: (optional) An alphanumeric value identifying the user profile.
    :attr str iam_id: (optional) An alphanumeric value identifying the user's IAM
          ID.
    :attr str realm: (optional) The realm of the user. The value is either `IBMid`
          or `SL`.
    :attr str user_id: (optional) The user ID used for login.
    :attr str firstname: (optional) The first name of the user.
    :attr str lastname: (optional) The last name of the user.
    :attr str state: (optional) The state of the user. Possible values are
          `PROCESSING`, `PENDING`, `ACTIVE`, `DISABLED_CLASSIC_INFRASTRUCTURE`, and
          `VPN_ONLY`.
    :attr str email: (optional) The email address of the user.
    :attr str phonenumber: (optional) The phone number of the user.
    :attr str altphonenumber: (optional) The alternative phone number of the user.
    :attr str photo: (optional) A link to a photo of the user.
    :attr str account_id: (optional) An alphanumeric value identifying the account
          ID.
    :attr str added_on: (optional) The timestamp for when the user was added to the
          account.
    """

    def __init__(
        self,
        *,
        id: str = None,
        iam_id: str = None,
        realm: str = None,
        user_id: str = None,
        firstname: str = None,
        lastname: str = None,
        state: str = None,
        email: str = None,
        phonenumber: str = None,
        altphonenumber: str = None,
        photo: str = None,
        account_id: str = None,
        added_on: str = None,
    ) -> None:
        """
        Initialize a UserProfile object.

        :param str id: (optional) An alphanumeric value identifying the user
               profile.
        :param str iam_id: (optional) An alphanumeric value identifying the user's
               IAM ID.
        :param str realm: (optional) The realm of the user. The value is either
               `IBMid` or `SL`.
        :param str user_id: (optional) The user ID used for login.
        :param str firstname: (optional) The first name of the user.
        :param str lastname: (optional) The last name of the user.
        :param str state: (optional) The state of the user. Possible values are
               `PROCESSING`, `PENDING`, `ACTIVE`, `DISABLED_CLASSIC_INFRASTRUCTURE`, and
               `VPN_ONLY`.
        :param str email: (optional) The email address of the user.
        :param str phonenumber: (optional) The phone number of the user.
        :param str altphonenumber: (optional) The alternative phone number of the
               user.
        :param str photo: (optional) A link to a photo of the user.
        :param str account_id: (optional) An alphanumeric value identifying the
               account ID.
        :param str added_on: (optional) The timestamp for when the user was added
               to the account.
        """
        self.id = id
        self.iam_id = iam_id
        self.realm = realm
        self.user_id = user_id
        self.firstname = firstname
        self.lastname = lastname
        self.state = state
        self.email = email
        self.phonenumber = phonenumber
        self.altphonenumber = altphonenumber
        self.photo = photo
        self.account_id = account_id
        self.added_on = added_on

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'UserProfile':
        """Initialize a UserProfile object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'iam_id' in _dict:
            args['iam_id'] = _dict.get('iam_id')
        if 'realm' in _dict:
            args['realm'] = _dict.get('realm')
        if 'user_id' in _dict:
            args['user_id'] = _dict.get('user_id')
        if 'firstname' in _dict:
            args['firstname'] = _dict.get('firstname')
        if 'lastname' in _dict:
            args['lastname'] = _dict.get('lastname')
        if 'state' in _dict:
            args['state'] = _dict.get('state')
        if 'email' in _dict:
            args['email'] = _dict.get('email')
        if 'phonenumber' in _dict:
            args['phonenumber'] = _dict.get('phonenumber')
        if 'altphonenumber' in _dict:
            args['altphonenumber'] = _dict.get('altphonenumber')
        if 'photo' in _dict:
            args['photo'] = _dict.get('photo')
        if 'account_id' in _dict:
            args['account_id'] = _dict.get('account_id')
        if 'added_on' in _dict:
            args['added_on'] = _dict.get('added_on')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a UserProfile object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'iam_id') and self.iam_id is not None:
            _dict['iam_id'] = self.iam_id
        if hasattr(self, 'realm') and self.realm is not None:
            _dict['realm'] = self.realm
        if hasattr(self, 'user_id') and self.user_id is not None:
            _dict['user_id'] = self.user_id
        if hasattr(self, 'firstname') and self.firstname is not None:
            _dict['firstname'] = self.firstname
        if hasattr(self, 'lastname') and self.lastname is not None:
            _dict['lastname'] = self.lastname
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        if hasattr(self, 'email') and self.email is not None:
            _dict['email'] = self.email
        if hasattr(self, 'phonenumber') and self.phonenumber is not None:
            _dict['phonenumber'] = self.phonenumber
        if hasattr(self, 'altphonenumber') and self.altphonenumber is not None:
            _dict['altphonenumber'] = self.altphonenumber
        if hasattr(self, 'photo') and self.photo is not None:
            _dict['photo'] = self.photo
        if hasattr(self, 'account_id') and self.account_id is not None:
            _dict['account_id'] = self.account_id
        if hasattr(self, 'added_on') and self.added_on is not None:
            _dict['added_on'] = self.added_on
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this UserProfile object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'UserProfile') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'UserProfile') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class UserSettings:
    """
    The user settings returned.

    :attr str language: (optional) The console UI language. By default, this field
          is empty.
    :attr str notification_language: (optional) The language for email and phone
          notifications. By default, this field is empty.
    :attr str allowed_ip_addresses: (optional) A comma-separated list of IP
          addresses.
    :attr bool self_manage: (optional) Whether user managed login is enabled. The
          default value is `false`.
    """

    def __init__(
        self,
        *,
        language: str = None,
        notification_language: str = None,
        allowed_ip_addresses: str = None,
        self_manage: bool = None,
    ) -> None:
        """
        Initialize a UserSettings object.

        :param str language: (optional) The console UI language. By default, this
               field is empty.
        :param str notification_language: (optional) The language for email and
               phone notifications. By default, this field is empty.
        :param str allowed_ip_addresses: (optional) A comma-separated list of IP
               addresses.
        :param bool self_manage: (optional) Whether user managed login is enabled.
               The default value is `false`.
        """
        self.language = language
        self.notification_language = notification_language
        self.allowed_ip_addresses = allowed_ip_addresses
        self.self_manage = self_manage

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'UserSettings':
        """Initialize a UserSettings object from a json dictionary."""
        args = {}
        if 'language' in _dict:
            args['language'] = _dict.get('language')
        if 'notification_language' in _dict:
            args['notification_language'] = _dict.get('notification_language')
        if 'allowed_ip_addresses' in _dict:
            args['allowed_ip_addresses'] = _dict.get('allowed_ip_addresses')
        if 'self_manage' in _dict:
            args['self_manage'] = _dict.get('self_manage')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a UserSettings object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'language') and self.language is not None:
            _dict['language'] = self.language
        if hasattr(self, 'notification_language') and self.notification_language is not None:
            _dict['notification_language'] = self.notification_language
        if hasattr(self, 'allowed_ip_addresses') and self.allowed_ip_addresses is not None:
            _dict['allowed_ip_addresses'] = self.allowed_ip_addresses
        if hasattr(self, 'self_manage') and self.self_manage is not None:
            _dict['self_manage'] = self.self_manage
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this UserSettings object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'UserSettings') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'UserSettings') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Attribute:
    """
    An attribute/value pair.

    :attr str name: (optional) The name of the attribute.
    :attr str value: (optional) The value of the attribute.
    """

    def __init__(
        self,
        *,
        name: str = None,
        value: str = None,
    ) -> None:
        """
        Initialize a Attribute object.

        :param str name: (optional) The name of the attribute.
        :param str value: (optional) The value of the attribute.
        """
        self.name = name
        self.value = value

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Attribute':
        """Initialize a Attribute object from a json dictionary."""
        args = {}
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        if 'value' in _dict:
            args['value'] = _dict.get('value')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Attribute object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'value') and self.value is not None:
            _dict['value'] = self.value
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Attribute object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'Attribute') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Attribute') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class InviteUser:
    """
    Invite a user.

    :attr str email: (optional) The email of the user to be invited.
    :attr str account_role: (optional) The account role of the user to be invited.
    """

    def __init__(
        self,
        *,
        email: str = None,
        account_role: str = None,
    ) -> None:
        """
        Initialize a InviteUser object.

        :param str email: (optional) The email of the user to be invited.
        :param str account_role: (optional) The account role of the user to be
               invited.
        """
        self.email = email
        self.account_role = account_role

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'InviteUser':
        """Initialize a InviteUser object from a json dictionary."""
        args = {}
        if 'email' in _dict:
            args['email'] = _dict.get('email')
        if 'account_role' in _dict:
            args['account_role'] = _dict.get('account_role')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a InviteUser object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'email') and self.email is not None:
            _dict['email'] = self.email
        if hasattr(self, 'account_role') and self.account_role is not None:
            _dict['account_role'] = self.account_role
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this InviteUser object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'InviteUser') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'InviteUser') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class InviteUserIamPolicy:
    """
    Invite a user to an IAM policy.

    :attr str type: The policy type. This can be either "access" or "authorization".
    :attr List[Role] roles: (optional) A list of IAM roles.
    :attr List[Resource] resources: (optional) A list of resources.
    """

    def __init__(
        self,
        type: str,
        *,
        roles: List['Role'] = None,
        resources: List['Resource'] = None,
    ) -> None:
        """
        Initialize a InviteUserIamPolicy object.

        :param str type: The policy type. This can be either "access" or
               "authorization".
        :param List[Role] roles: (optional) A list of IAM roles.
        :param List[Resource] resources: (optional) A list of resources.
        """
        self.type = type
        self.roles = roles
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'InviteUserIamPolicy':
        """Initialize a InviteUserIamPolicy object from a json dictionary."""
        args = {}
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        else:
            raise ValueError('Required property \'type\' not present in InviteUserIamPolicy JSON')
        if 'roles' in _dict:
            args['roles'] = [Role.from_dict(v) for v in _dict.get('roles')]
        if 'resources' in _dict:
            args['resources'] = [Resource.from_dict(v) for v in _dict.get('resources')]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a InviteUserIamPolicy object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'roles') and self.roles is not None:
            roles_list = []
            for v in self.roles:
                if isinstance(v, dict):
                    roles_list.append(v)
                else:
                    roles_list.append(v.to_dict())
            _dict['roles'] = roles_list
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this InviteUserIamPolicy object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'InviteUserIamPolicy') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'InviteUserIamPolicy') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Resource:
    """
    A collection of attribute value pairs.

    :attr List[Attribute] attributes: (optional) A list of IAM attributes.
    """

    def __init__(
        self,
        *,
        attributes: List['Attribute'] = None,
    ) -> None:
        """
        Initialize a Resource object.

        :param List[Attribute] attributes: (optional) A list of IAM attributes.
        """
        self.attributes = attributes

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Resource':
        """Initialize a Resource object from a json dictionary."""
        args = {}
        if 'attributes' in _dict:
            args['attributes'] = [Attribute.from_dict(v) for v in _dict.get('attributes')]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Resource object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'attributes') and self.attributes is not None:
            attributes_list = []
            for v in self.attributes:
                if isinstance(v, dict):
                    attributes_list.append(v)
                else:
                    attributes_list.append(v.to_dict())
            _dict['attributes'] = attributes_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Resource object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'Resource') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Resource') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Role:
    """
    The role of an IAM policy.

    :attr str role_id: (optional) An alphanumeric value identifying the origin.
    """

    def __init__(
        self,
        *,
        role_id: str = None,
    ) -> None:
        """
        Initialize a Role object.

        :param str role_id: (optional) An alphanumeric value identifying the
               origin.
        """
        self.role_id = role_id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Role':
        """Initialize a Role object from a json dictionary."""
        args = {}
        if 'role_id' in _dict:
            args['role_id'] = _dict.get('role_id')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Role object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'role_id') and self.role_id is not None:
            _dict['role_id'] = self.role_id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Role object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'Role') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Role') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


##############################################################################
# Pagers
##############################################################################


class UsersPager:
    """
    UsersPager can be used to simplify the use of the "list_users" method.
    """

    def __init__(
        self,
        *,
        client: UserManagementV1,
        account_id: str,
        limit: int = None,
        include_settings: bool = None,
        search: str = None,
        user_id: str = None,
    ) -> None:
        """
        Initialize a UsersPager object.
        :param str account_id: The account ID of the specified user.
        :param int limit: (optional) The number of results to be returned.
        :param bool include_settings: (optional) The user settings to be returned.
               Set to true to view language, allowed IP address, and authentication
               settings.
        :param str search: (optional) The desired search results to be returned. To
               view the list of users with the additional search filter, use the following
               query options: `firstname`, `lastname`, `email`, `state`, `substate`,
               `iam_id`, `realm`, and `userId`. HTML URL encoding for the search query and
               `:` must be used. For example, search=state%3AINVALID returns a list of
               invalid users. Multiple search queries can be combined to obtain `OR`
               results using `,` operator (not URL encoded). For example,
               search=state%3AINVALID,email%3Amail.test.ibm.com.
        :param str user_id: (optional) Filter users based on their user ID.
        """
        self._has_next = True
        self._client = client
        self._page_context = {'next': None}
        self._account_id = account_id
        self._limit = limit
        self._include_settings = include_settings
        self._search = search
        self._user_id = user_id

    def has_next(self) -> bool:
        """
        Returns true if there are potentially more results to be retrieved.
        """
        return self._has_next

    def get_next(self) -> List[dict]:
        """
        Returns the next page of results.
        :return: A List[dict], where each element is a dict that represents an instance of UserProfile.
        :rtype: List[dict]
        """
        if not self.has_next():
            raise StopIteration(message='No more results available')

        result = self._client.list_users(
            account_id=self._account_id,
            limit=self._limit,
            include_settings=self._include_settings,
            search=self._search,
            user_id=self._user_id,
            start=self._page_context.get('next'),
        ).get_result()

        next = None
        next_page_link = result.get('next_url')
        if next_page_link is not None:
            next = get_query_param(next_page_link, '_start')
        self._page_context['next'] = next
        if next is None:
            self._has_next = False

        return result.get('resources')

    def get_all(self) -> List[dict]:
        """
        Returns all results by invoking get_next() repeatedly
        until all pages of results have been retrieved.
        :return: A List[dict], where each element is a dict that represents an instance of UserProfile.
        :rtype: List[dict]
        """
        results = []
        while self.has_next():
            next_page = self.get_next()
            results.extend(next_page)
        return results



# coding: utf-8

# (C) Copyright IBM Corp. 2024.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# IBM OpenAPI SDK Code Generator Version: 3.85.0-75c38f8f-20240206-210220

"""
Usage reports for IBM Cloud partner entities

API Version: 1.0.0
"""

from enum import Enum
from typing import Dict, List, Optional
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################


class PartnerUsageReportsV1(BaseService):
    """The Partner Usage Reports V1 service."""

    DEFAULT_SERVICE_URL = 'https://partner.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'partner_usage_reports'

    @classmethod
    def new_instance(
        cls,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> 'PartnerUsageReportsV1':
        """
        Return a new client for the Partner Usage Reports service using the
               specified parameters and external configuration.
        """
        authenticator = get_authenticator_from_environment(service_name)
        service = cls(authenticator)
        service.configure_service(service_name)
        return service

    def __init__(
        self,
        authenticator: Authenticator = None,
    ) -> None:
        """
        Construct a new client for the Partner Usage Reports service.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/main/README.md
               about initializing the authenticator of your choice.
        """
        BaseService.__init__(self, service_url=self.DEFAULT_SERVICE_URL, authenticator=authenticator)

    #########################
    # Partner Usage Reports
    #########################

    def get_resource_usage_report(
        self,
        partner_id: str,
        *,
        reseller_id: Optional[str] = None,
        customer_id: Optional[str] = None,
        children: Optional[bool] = None,
        month: Optional[str] = None,
        viewpoint: Optional[str] = None,
        recurse: Optional[bool] = None,
        limit: Optional[int] = None,
        offset: Optional[str] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get partner resource usage report.

        Returns the summary for the partner for a given month. Partner billing managers
        are authorized to access this report.

        :param str partner_id: Enterprise ID of the distributor or reseller for
               which the report is requested.
        :param str reseller_id: (optional) Enterprise ID of the reseller for which
               the report is requested. This parameter cannot be used along with
               `customer_id` query parameter.
        :param str customer_id: (optional) Enterprise ID of the child customer for
               which the report is requested. This parameter cannot be used along with
               `reseller_id` query parameter.
        :param bool children: (optional) Get report rolled-up to the direct
               children of the requested entity. Defaults to false. This parameter cannot
               be used along with `customer_id` query parameter.
        :param str month: (optional) The billing month for which the usage report
               is requested. Format is `yyyy-mm`. Defaults to current month.
        :param str viewpoint: (optional) Enables partner to view the cost of
               provisioned services as applicable at each level of the hierarchy. Defaults
               to the type of the calling partner. The valid values are `DISTRIBUTOR`,
               `RESELLER` and `END_CUSTOMER`.
        :param bool recurse: (optional) Get usage report rolled-up to the end
               customers of the requested entity. Defaults to false. This parameter cannot
               be used along with `reseller_id` query parameter or `customer_id` query
               parameter.
        :param int limit: (optional) Number of usage records to be returned. The
               default value is 30. Maximum value is 200.
        :param str offset: (optional) An opaque value representing the offset of
               the first item to be returned by a search query. If not specified, then the
               first page of results is returned. To retrieve the next page of search
               results, use the 'offset' query parameter value within the 'next.href' URL
               found within a prior search query response.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `PartnerUsageReportSummary` object
        """

        if not partner_id:
            raise ValueError('partner_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_resource_usage_report',
        )
        headers.update(sdk_headers)

        params = {
            'partner_id': partner_id,
            'reseller_id': reseller_id,
            'customer_id': customer_id,
            'children': children,
            'month': month,
            'viewpoint': viewpoint,
            'recurse': recurse,
            'limit': limit,
            'offset': offset,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/v1/resource-usage-reports'
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response


class GetResourceUsageReportEnums:
    """
    Enums for get_resource_usage_report parameters.
    """

    class Viewpoint(str, Enum):
        """
        Enables partner to view the cost of provisioned services as applicable at each
        level of the hierarchy. Defaults to the type of the calling partner. The valid
        values are `DISTRIBUTOR`, `RESELLER` and `END_CUSTOMER`.
        """

        DISTRIBUTOR = 'DISTRIBUTOR'
        RESELLER = 'RESELLER'
        END_CUSTOMER = 'END_CUSTOMER'


##############################################################################
# Models
##############################################################################


class MetricUsage:
    """
    An object that represents a metric.

    :param str metric: The name of the metric.
    :param str unit: A unit to qualify the quantity.
    :param float quantity: The aggregated value for the metric.
    :param float rateable_quantity: The quantity that is used for calculating
          charges.
    :param float cost: The cost that was incurred by the metric.
    :param float rated_cost: The pre-discounted cost that was incurred by the
          metric.
    :param List[dict] price: (optional) The price with which cost was calculated.
    """

    def __init__(
        self,
        metric: str,
        unit: str,
        quantity: float,
        rateable_quantity: float,
        cost: float,
        rated_cost: float,
        *,
        price: Optional[List[dict]] = None,
    ) -> None:
        """
        Initialize a MetricUsage object.

        :param str metric: The name of the metric.
        :param str unit: A unit to qualify the quantity.
        :param float quantity: The aggregated value for the metric.
        :param float rateable_quantity: The quantity that is used for calculating
               charges.
        :param float cost: The cost that was incurred by the metric.
        :param float rated_cost: The pre-discounted cost that was incurred by the
               metric.
        :param List[dict] price: (optional) The price with which cost was
               calculated.
        """
        self.metric = metric
        self.unit = unit
        self.quantity = quantity
        self.rateable_quantity = rateable_quantity
        self.cost = cost
        self.rated_cost = rated_cost
        self.price = price

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'MetricUsage':
        """Initialize a MetricUsage object from a json dictionary."""
        args = {}
        if (metric := _dict.get('metric')) is not None:
            args['metric'] = metric
        else:
            raise ValueError('Required property \'metric\' not present in MetricUsage JSON')
        if (unit := _dict.get('unit')) is not None:
            args['unit'] = unit
        else:
            raise ValueError('Required property \'unit\' not present in MetricUsage JSON')
        if (quantity := _dict.get('quantity')) is not None:
            args['quantity'] = quantity
        else:
            raise ValueError('Required property \'quantity\' not present in MetricUsage JSON')
        if (rateable_quantity := _dict.get('rateable_quantity')) is not None:
            args['rateable_quantity'] = rateable_quantity
        else:
            raise ValueError('Required property \'rateable_quantity\' not present in MetricUsage JSON')
        if (cost := _dict.get('cost')) is not None:
            args['cost'] = cost
        else:
            raise ValueError('Required property \'cost\' not present in MetricUsage JSON')
        if (rated_cost := _dict.get('rated_cost')) is not None:
            args['rated_cost'] = rated_cost
        else:
            raise ValueError('Required property \'rated_cost\' not present in MetricUsage JSON')
        if (price := _dict.get('price')) is not None:
            args['price'] = price
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a MetricUsage object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'metric') and self.metric is not None:
            _dict['metric'] = self.metric
        if hasattr(self, 'unit') and self.unit is not None:
            _dict['unit'] = self.unit
        if hasattr(self, 'quantity') and self.quantity is not None:
            _dict['quantity'] = self.quantity
        if hasattr(self, 'rateable_quantity') and self.rateable_quantity is not None:
            _dict['rateable_quantity'] = self.rateable_quantity
        if hasattr(self, 'cost') and self.cost is not None:
            _dict['cost'] = self.cost
        if hasattr(self, 'rated_cost') and self.rated_cost is not None:
            _dict['rated_cost'] = self.rated_cost
        if hasattr(self, 'price') and self.price is not None:
            _dict['price'] = self.price
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this MetricUsage object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'MetricUsage') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'MetricUsage') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class PartnerUsageReportSummaryFirst:
    """
    The link to the first page of the search query.

    :param str href: (optional) A link to a page of query results.
    """

    def __init__(
        self,
        *,
        href: Optional[str] = None,
    ) -> None:
        """
        Initialize a PartnerUsageReportSummaryFirst object.

        :param str href: (optional) A link to a page of query results.
        """
        self.href = href

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PartnerUsageReportSummaryFirst':
        """Initialize a PartnerUsageReportSummaryFirst object from a json dictionary."""
        args = {}
        if (href := _dict.get('href')) is not None:
            args['href'] = href
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PartnerUsageReportSummaryFirst object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'href') and self.href is not None:
            _dict['href'] = self.href
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PartnerUsageReportSummaryFirst object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PartnerUsageReportSummaryFirst') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PartnerUsageReportSummaryFirst') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class PartnerUsageReportSummaryNext:
    """
    The link to the next page of the search query.

    :param str href: (optional) A link to a page of query results.
    :param str offset: (optional) The value of the `_start` query parameter to fetch
          the next page.
    """

    def __init__(
        self,
        *,
        href: Optional[str] = None,
        offset: Optional[str] = None,
    ) -> None:
        """
        Initialize a PartnerUsageReportSummaryNext object.

        :param str href: (optional) A link to a page of query results.
        :param str offset: (optional) The value of the `_start` query parameter to
               fetch the next page.
        """
        self.href = href
        self.offset = offset

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PartnerUsageReportSummaryNext':
        """Initialize a PartnerUsageReportSummaryNext object from a json dictionary."""
        args = {}
        if (href := _dict.get('href')) is not None:
            args['href'] = href
        if (offset := _dict.get('offset')) is not None:
            args['offset'] = offset
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PartnerUsageReportSummaryNext object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'href') and self.href is not None:
            _dict['href'] = self.href
        if hasattr(self, 'offset') and self.offset is not None:
            _dict['offset'] = self.offset
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PartnerUsageReportSummaryNext object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PartnerUsageReportSummaryNext') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PartnerUsageReportSummaryNext') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class PartnerUsageReport:
    """
    Aggregated usage report of a partner.

    :param str entity_id: (optional) The ID of the entity.
    :param str entity_type: (optional) The entity type.
    :param str entity_crn: (optional) The Cloud Resource Name (CRN) of the entity
          towards which the resource usages were rolled up.
    :param str entity_name: (optional) A user-defined name for the entity, such as
          the enterprise name or account group name.
    :param str entity_partner_type: (optional) Role of the `entity_id` for which the
          usage report is fetched.
    :param str viewpoint: (optional) Enables partner to view the cost of provisioned
          services as applicable at each level of the hierarchy.
    :param str month: (optional) The billing month for which the usage report is
          requested. Format is yyyy-mm.
    :param str currency_code: (optional) The currency code of the billing unit.
    :param str country_code: (optional) The country code of the billing unit.
    :param float billable_cost: (optional) Billable charges that are aggregated from
          all entities in the report.
    :param float billable_rated_cost: (optional) Aggregated billable charges before
          discounts.
    :param float non_billable_cost: (optional) Non-billable charges that are
          aggregated from all entities in the report.
    :param float non_billable_rated_cost: (optional) Aggregated non-billable charges
          before discounts.
    :param List[ResourceUsage] resources: (optional)
    """

    def __init__(
        self,
        *,
        entity_id: Optional[str] = None,
        entity_type: Optional[str] = None,
        entity_crn: Optional[str] = None,
        entity_name: Optional[str] = None,
        entity_partner_type: Optional[str] = None,
        viewpoint: Optional[str] = None,
        month: Optional[str] = None,
        currency_code: Optional[str] = None,
        country_code: Optional[str] = None,
        billable_cost: Optional[float] = None,
        billable_rated_cost: Optional[float] = None,
        non_billable_cost: Optional[float] = None,
        non_billable_rated_cost: Optional[float] = None,
        resources: Optional[List['ResourceUsage']] = None,
    ) -> None:
        """
        Initialize a PartnerUsageReport object.

        :param str entity_id: (optional) The ID of the entity.
        :param str entity_type: (optional) The entity type.
        :param str entity_crn: (optional) The Cloud Resource Name (CRN) of the
               entity towards which the resource usages were rolled up.
        :param str entity_name: (optional) A user-defined name for the entity, such
               as the enterprise name or account group name.
        :param str entity_partner_type: (optional) Role of the `entity_id` for
               which the usage report is fetched.
        :param str viewpoint: (optional) Enables partner to view the cost of
               provisioned services as applicable at each level of the hierarchy.
        :param str month: (optional) The billing month for which the usage report
               is requested. Format is yyyy-mm.
        :param str currency_code: (optional) The currency code of the billing unit.
        :param str country_code: (optional) The country code of the billing unit.
        :param float billable_cost: (optional) Billable charges that are aggregated
               from all entities in the report.
        :param float billable_rated_cost: (optional) Aggregated billable charges
               before discounts.
        :param float non_billable_cost: (optional) Non-billable charges that are
               aggregated from all entities in the report.
        :param float non_billable_rated_cost: (optional) Aggregated non-billable
               charges before discounts.
        :param List[ResourceUsage] resources: (optional)
        """
        self.entity_id = entity_id
        self.entity_type = entity_type
        self.entity_crn = entity_crn
        self.entity_name = entity_name
        self.entity_partner_type = entity_partner_type
        self.viewpoint = viewpoint
        self.month = month
        self.currency_code = currency_code
        self.country_code = country_code
        self.billable_cost = billable_cost
        self.billable_rated_cost = billable_rated_cost
        self.non_billable_cost = non_billable_cost
        self.non_billable_rated_cost = non_billable_rated_cost
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PartnerUsageReport':
        """Initialize a PartnerUsageReport object from a json dictionary."""
        args = {}
        if (entity_id := _dict.get('entity_id')) is not None:
            args['entity_id'] = entity_id
        if (entity_type := _dict.get('entity_type')) is not None:
            args['entity_type'] = entity_type
        if (entity_crn := _dict.get('entity_crn')) is not None:
            args['entity_crn'] = entity_crn
        if (entity_name := _dict.get('entity_name')) is not None:
            args['entity_name'] = entity_name
        if (entity_partner_type := _dict.get('entity_partner_type')) is not None:
            args['entity_partner_type'] = entity_partner_type
        if (viewpoint := _dict.get('viewpoint')) is not None:
            args['viewpoint'] = viewpoint
        if (month := _dict.get('month')) is not None:
            args['month'] = month
        if (currency_code := _dict.get('currency_code')) is not None:
            args['currency_code'] = currency_code
        if (country_code := _dict.get('country_code')) is not None:
            args['country_code'] = country_code
        if (billable_cost := _dict.get('billable_cost')) is not None:
            args['billable_cost'] = billable_cost
        if (billable_rated_cost := _dict.get('billable_rated_cost')) is not None:
            args['billable_rated_cost'] = billable_rated_cost
        if (non_billable_cost := _dict.get('non_billable_cost')) is not None:
            args['non_billable_cost'] = non_billable_cost
        if (non_billable_rated_cost := _dict.get('non_billable_rated_cost')) is not None:
            args['non_billable_rated_cost'] = non_billable_rated_cost
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = [ResourceUsage.from_dict(v) for v in resources]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PartnerUsageReport object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'entity_id') and self.entity_id is not None:
            _dict['entity_id'] = self.entity_id
        if hasattr(self, 'entity_type') and self.entity_type is not None:
            _dict['entity_type'] = self.entity_type
        if hasattr(self, 'entity_crn') and self.entity_crn is not None:
            _dict['entity_crn'] = self.entity_crn
        if hasattr(self, 'entity_name') and self.entity_name is not None:
            _dict['entity_name'] = self.entity_name
        if hasattr(self, 'entity_partner_type') and self.entity_partner_type is not None:
            _dict['entity_partner_type'] = self.entity_partner_type
        if hasattr(self, 'viewpoint') and self.viewpoint is not None:
            _dict['viewpoint'] = self.viewpoint
        if hasattr(self, 'month') and self.month is not None:
            _dict['month'] = self.month
        if hasattr(self, 'currency_code') and self.currency_code is not None:
            _dict['currency_code'] = self.currency_code
        if hasattr(self, 'country_code') and self.country_code is not None:
            _dict['country_code'] = self.country_code
        if hasattr(self, 'billable_cost') and self.billable_cost is not None:
            _dict['billable_cost'] = self.billable_cost
        if hasattr(self, 'billable_rated_cost') and self.billable_rated_cost is not None:
            _dict['billable_rated_cost'] = self.billable_rated_cost
        if hasattr(self, 'non_billable_cost') and self.non_billable_cost is not None:
            _dict['non_billable_cost'] = self.non_billable_cost
        if hasattr(self, 'non_billable_rated_cost') and self.non_billable_rated_cost is not None:
            _dict['non_billable_rated_cost'] = self.non_billable_rated_cost
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PartnerUsageReport object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PartnerUsageReport') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PartnerUsageReport') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class PartnerUsageReportSummary:
    """
    The aggregated partner usage report.

    :param int limit: (optional) The maximum number of usage records in the
          response.
    :param PartnerUsageReportSummaryFirst first: (optional) The link to the first
          page of the search query.
    :param PartnerUsageReportSummaryNext next: (optional) The link to the next page
          of the search query.
    :param List[PartnerUsageReport] reports: (optional) Aggregated usage report of
          all requested partners.
    """

    def __init__(
        self,
        *,
        limit: Optional[int] = None,
        first: Optional['PartnerUsageReportSummaryFirst'] = None,
        next: Optional['PartnerUsageReportSummaryNext'] = None,
        reports: Optional[List['PartnerUsageReport']] = None,
    ) -> None:
        """
        Initialize a PartnerUsageReportSummary object.

        :param int limit: (optional) The maximum number of usage records in the
               response.
        :param PartnerUsageReportSummaryFirst first: (optional) The link to the
               first page of the search query.
        :param PartnerUsageReportSummaryNext next: (optional) The link to the next
               page of the search query.
        :param List[PartnerUsageReport] reports: (optional) Aggregated usage report
               of all requested partners.
        """
        self.limit = limit
        self.first = first
        self.next = next
        self.reports = reports

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PartnerUsageReportSummary':
        """Initialize a PartnerUsageReportSummary object from a json dictionary."""
        args = {}
        if (limit := _dict.get('limit')) is not None:
            args['limit'] = limit
        if (first := _dict.get('first')) is not None:
            args['first'] = PartnerUsageReportSummaryFirst.from_dict(first)
        if (next := _dict.get('next')) is not None:
            args['next'] = PartnerUsageReportSummaryNext.from_dict(next)
        if (reports := _dict.get('reports')) is not None:
            args['reports'] = [PartnerUsageReport.from_dict(v) for v in reports]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PartnerUsageReportSummary object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'limit') and self.limit is not None:
            _dict['limit'] = self.limit
        if hasattr(self, 'first') and self.first is not None:
            if isinstance(self.first, dict):
                _dict['first'] = self.first
            else:
                _dict['first'] = self.first.to_dict()
        if hasattr(self, 'next') and self.next is not None:
            if isinstance(self.next, dict):
                _dict['next'] = self.next
            else:
                _dict['next'] = self.next.to_dict()
        if hasattr(self, 'reports') and self.reports is not None:
            reports_list = []
            for v in self.reports:
                if isinstance(v, dict):
                    reports_list.append(v)
                else:
                    reports_list.append(v.to_dict())
            _dict['reports'] = reports_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PartnerUsageReportSummary object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PartnerUsageReportSummary') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PartnerUsageReportSummary') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class PlanUsage:
    """
    Aggregated values for the plan.

    :param str plan_id: The ID of the plan.
    :param str pricing_region: (optional) The pricing region for the plan.
    :param str pricing_plan_id: (optional) The pricing plan with which the usage was
          rated.
    :param bool billable: Whether the plan charges are billed to the customer.
    :param float cost: The total cost that was incurred by the plan.
    :param float rated_cost: The total pre-discounted cost that was incurred by the
          plan.
    :param List[MetricUsage] usage: All of the metrics in the plan.
    """

    def __init__(
        self,
        plan_id: str,
        billable: bool,
        cost: float,
        rated_cost: float,
        usage: List['MetricUsage'],
        *,
        pricing_region: Optional[str] = None,
        pricing_plan_id: Optional[str] = None,
    ) -> None:
        """
        Initialize a PlanUsage object.

        :param str plan_id: The ID of the plan.
        :param bool billable: Whether the plan charges are billed to the customer.
        :param float cost: The total cost that was incurred by the plan.
        :param float rated_cost: The total pre-discounted cost that was incurred by
               the plan.
        :param List[MetricUsage] usage: All of the metrics in the plan.
        :param str pricing_region: (optional) The pricing region for the plan.
        :param str pricing_plan_id: (optional) The pricing plan with which the
               usage was rated.
        """
        self.plan_id = plan_id
        self.pricing_region = pricing_region
        self.pricing_plan_id = pricing_plan_id
        self.billable = billable
        self.cost = cost
        self.rated_cost = rated_cost
        self.usage = usage

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PlanUsage':
        """Initialize a PlanUsage object from a json dictionary."""
        args = {}
        if (plan_id := _dict.get('plan_id')) is not None:
            args['plan_id'] = plan_id
        else:
            raise ValueError('Required property \'plan_id\' not present in PlanUsage JSON')
        if (pricing_region := _dict.get('pricing_region')) is not None:
            args['pricing_region'] = pricing_region
        if (pricing_plan_id := _dict.get('pricing_plan_id')) is not None:
            args['pricing_plan_id'] = pricing_plan_id
        if (billable := _dict.get('billable')) is not None:
            args['billable'] = billable
        else:
            raise ValueError('Required property \'billable\' not present in PlanUsage JSON')
        if (cost := _dict.get('cost')) is not None:
            args['cost'] = cost
        else:
            raise ValueError('Required property \'cost\' not present in PlanUsage JSON')
        if (rated_cost := _dict.get('rated_cost')) is not None:
            args['rated_cost'] = rated_cost
        else:
            raise ValueError('Required property \'rated_cost\' not present in PlanUsage JSON')
        if (usage := _dict.get('usage')) is not None:
            args['usage'] = [MetricUsage.from_dict(v) for v in usage]
        else:
            raise ValueError('Required property \'usage\' not present in PlanUsage JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PlanUsage object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'plan_id') and self.plan_id is not None:
            _dict['plan_id'] = self.plan_id
        if hasattr(self, 'pricing_region') and self.pricing_region is not None:
            _dict['pricing_region'] = self.pricing_region
        if hasattr(self, 'pricing_plan_id') and self.pricing_plan_id is not None:
            _dict['pricing_plan_id'] = self.pricing_plan_id
        if hasattr(self, 'billable') and self.billable is not None:
            _dict['billable'] = self.billable
        if hasattr(self, 'cost') and self.cost is not None:
            _dict['cost'] = self.cost
        if hasattr(self, 'rated_cost') and self.rated_cost is not None:
            _dict['rated_cost'] = self.rated_cost
        if hasattr(self, 'usage') and self.usage is not None:
            usage_list = []
            for v in self.usage:
                if isinstance(v, dict):
                    usage_list.append(v)
                else:
                    usage_list.append(v.to_dict())
            _dict['usage'] = usage_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PlanUsage object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PlanUsage') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PlanUsage') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ResourceUsage:
    """
    A container for all the plans in the resource.

    :param str resource_id: The ID of the resource.
    :param str resource_name: (optional) The name of the resource.
    :param float billable_cost: The billable charges for the partner.
    :param float billable_rated_cost: The pre-discounted billable charges for the
          partner.
    :param float non_billable_cost: The non-billable charges for the partner.
    :param float non_billable_rated_cost: The pre-discounted, non-billable charges
          for the partner.
    :param List[PlanUsage] plans: All of the plans in the resource.
    """

    def __init__(
        self,
        resource_id: str,
        billable_cost: float,
        billable_rated_cost: float,
        non_billable_cost: float,
        non_billable_rated_cost: float,
        plans: List['PlanUsage'],
        *,
        resource_name: Optional[str] = None,
    ) -> None:
        """
        Initialize a ResourceUsage object.

        :param str resource_id: The ID of the resource.
        :param float billable_cost: The billable charges for the partner.
        :param float billable_rated_cost: The pre-discounted billable charges for
               the partner.
        :param float non_billable_cost: The non-billable charges for the partner.
        :param float non_billable_rated_cost: The pre-discounted, non-billable
               charges for the partner.
        :param List[PlanUsage] plans: All of the plans in the resource.
        :param str resource_name: (optional) The name of the resource.
        """
        self.resource_id = resource_id
        self.resource_name = resource_name
        self.billable_cost = billable_cost
        self.billable_rated_cost = billable_rated_cost
        self.non_billable_cost = non_billable_cost
        self.non_billable_rated_cost = non_billable_rated_cost
        self.plans = plans

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResourceUsage':
        """Initialize a ResourceUsage object from a json dictionary."""
        args = {}
        if (resource_id := _dict.get('resource_id')) is not None:
            args['resource_id'] = resource_id
        else:
            raise ValueError('Required property \'resource_id\' not present in ResourceUsage JSON')
        if (resource_name := _dict.get('resource_name')) is not None:
            args['resource_name'] = resource_name
        if (billable_cost := _dict.get('billable_cost')) is not None:
            args['billable_cost'] = billable_cost
        else:
            raise ValueError('Required property \'billable_cost\' not present in ResourceUsage JSON')
        if (billable_rated_cost := _dict.get('billable_rated_cost')) is not None:
            args['billable_rated_cost'] = billable_rated_cost
        else:
            raise ValueError('Required property \'billable_rated_cost\' not present in ResourceUsage JSON')
        if (non_billable_cost := _dict.get('non_billable_cost')) is not None:
            args['non_billable_cost'] = non_billable_cost
        else:
            raise ValueError('Required property \'non_billable_cost\' not present in ResourceUsage JSON')
        if (non_billable_rated_cost := _dict.get('non_billable_rated_cost')) is not None:
            args['non_billable_rated_cost'] = non_billable_rated_cost
        else:
            raise ValueError('Required property \'non_billable_rated_cost\' not present in ResourceUsage JSON')
        if (plans := _dict.get('plans')) is not None:
            args['plans'] = [PlanUsage.from_dict(v) for v in plans]
        else:
            raise ValueError('Required property \'plans\' not present in ResourceUsage JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResourceUsage object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'resource_id') and self.resource_id is not None:
            _dict['resource_id'] = self.resource_id
        if hasattr(self, 'resource_name') and self.resource_name is not None:
            _dict['resource_name'] = self.resource_name
        if hasattr(self, 'billable_cost') and self.billable_cost is not None:
            _dict['billable_cost'] = self.billable_cost
        if hasattr(self, 'billable_rated_cost') and self.billable_rated_cost is not None:
            _dict['billable_rated_cost'] = self.billable_rated_cost
        if hasattr(self, 'non_billable_cost') and self.non_billable_cost is not None:
            _dict['non_billable_cost'] = self.non_billable_cost
        if hasattr(self, 'non_billable_rated_cost') and self.non_billable_rated_cost is not None:
            _dict['non_billable_rated_cost'] = self.non_billable_rated_cost
        if hasattr(self, 'plans') and self.plans is not None:
            plans_list = []
            for v in self.plans:
                if isinstance(v, dict):
                    plans_list.append(v)
                else:
                    plans_list.append(v.to_dict())
            _dict['plans'] = plans_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResourceUsage object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResourceUsage') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResourceUsage') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


##############################################################################
# Pagers
##############################################################################


class GetResourceUsageReportPager:
    """
    GetResourceUsageReportPager can be used to simplify the use of the "get_resource_usage_report" method.
    """

    def __init__(
        self,
        *,
        client: PartnerUsageReportsV1,
        partner_id: str,
        reseller_id: str = None,
        customer_id: str = None,
        children: bool = None,
        month: str = None,
        viewpoint: str = None,
        recurse: bool = None,
        limit: int = None,
    ) -> None:
        """
        Initialize a GetResourceUsageReportPager object.
        :param str partner_id: Enterprise ID of the distributor or reseller for
               which the report is requested.
        :param str reseller_id: (optional) Enterprise ID of the reseller for which
               the report is requested. This parameter cannot be used along with
               `customer_id` query parameter.
        :param str customer_id: (optional) Enterprise ID of the child customer for
               which the report is requested. This parameter cannot be used along with
               `reseller_id` query parameter.
        :param bool children: (optional) Get report rolled-up to the direct
               children of the requested entity. Defaults to false. This parameter cannot
               be used along with `customer_id` query parameter.
        :param str month: (optional) The billing month for which the usage report
               is requested. Format is `yyyy-mm`. Defaults to current month.
        :param str viewpoint: (optional) Enables partner to view the cost of
               provisioned services as applicable at each level of the hierarchy. Defaults
               to the type of the calling partner. The valid values are `DISTRIBUTOR`,
               `RESELLER` and `END_CUSTOMER`.
        :param bool recurse: (optional) Get usage report rolled-up to the end
               customers of the requested entity. Defaults to false. This parameter cannot
               be used along with `reseller_id` query parameter or `customer_id` query
               parameter.
        :param int limit: (optional) Number of usage records to be returned. The
               default value is 30. Maximum value is 200.
        """
        self._has_next = True
        self._client = client
        self._page_context = {'next': None}
        self._partner_id = partner_id
        self._reseller_id = reseller_id
        self._customer_id = customer_id
        self._children = children
        self._month = month
        self._viewpoint = viewpoint
        self._recurse = recurse
        self._limit = limit

    def has_next(self) -> bool:
        """
        Returns true if there are potentially more results to be retrieved.
        """
        return self._has_next

    def get_next(self) -> List[dict]:
        """
        Returns the next page of results.
        :return: A List[dict], where each element is a dict that represents an instance of PartnerUsageReport.
        :rtype: List[dict]
        """
        if not self.has_next():
            raise StopIteration(message='No more results available')

        result = self._client.get_resource_usage_report(
            partner_id=self._partner_id,
            reseller_id=self._reseller_id,
            customer_id=self._customer_id,
            children=self._children,
            month=self._month,
            viewpoint=self._viewpoint,
            recurse=self._recurse,
            limit=self._limit,
            offset=self._page_context.get('next'),
        ).get_result()

        next = None
        next_page_link = result.get('next')
        if next_page_link is not None:
            next = next_page_link.get('offset')
        self._page_context['next'] = next
        if next is None:
            self._has_next = False

        return result.get('reports')

    def get_all(self) -> List[dict]:
        """
        Returns all results by invoking get_next() repeatedly
        until all pages of results have been retrieved.
        :return: A List[dict], where each element is a dict that represents an instance of PartnerUsageReport.
        :rtype: List[dict]
        """
        results = []
        while self.has_next():
            next_page = self.get_next()
            results.extend(next_page)
        return results



# coding: utf-8

# (C) Copyright IBM Corp. 2021.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# IBM OpenAPI SDK Code Generator Version: 3.33.0-caf29bd0-20210603-225214

"""
API docs for IBM Cloud Shell repository
"""

from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import convert_model

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################


class IbmCloudShellV1(BaseService):
    """The IBM Cloud Shell V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.shell.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'ibm_cloud_shell'

    @classmethod
    def new_instance(
        cls,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> 'IbmCloudShellV1':
        """
        Return a new client for the IBM Cloud Shell service using the specified
               parameters and external configuration.
        """
        authenticator = get_authenticator_from_environment(service_name)
        service = cls(authenticator)
        service.configure_service(service_name)
        return service

    def __init__(
        self,
        authenticator: Authenticator = None,
    ) -> None:
        """
        Construct a new client for the IBM Cloud Shell service.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        BaseService.__init__(self, service_url=self.DEFAULT_SERVICE_URL, authenticator=authenticator)

    #########################
    # account_settings
    #########################

    def get_account_settings(self, account_id: str, **kwargs) -> DetailedResponse:
        """
        Get account settings.

        Retrieve account settings for the given account ID. Call this method to get
        details about a particular account setting, whether Cloud Shell is enabled, the
        list of enabled regions and the list of enabled features. Users need to be an
        account owner or users need to be assigned an IAM policy with the Administrator
        role for the Cloud Shell account management service.

        :param str account_id: The account ID in which the account settings belong
               to.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `AccountSettings` object
        """

        if account_id is None:
            raise ValueError('account_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V1', operation_id='get_account_settings'
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_id']
        path_param_values = self.encode_path_vars(account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/api/v1/user/accounts/{account_id}/settings'.format(**path_param_dict)
        request = self.prepare_request(method='GET', url=url, headers=headers)

        response = self.send(request)
        return response

    def update_account_settings(
        self,
        account_id: str,
        *,
        rev: str = None,
        default_enable_new_features: bool = None,
        default_enable_new_regions: bool = None,
        enabled: bool = None,
        features: List['Feature'] = None,
        regions: List['RegionSetting'] = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update account settings.

        Update account settings for the given account ID. Call this method to update
        account settings configuration, you can enable or disable Cloud Shell, enable or
        disable available regions and enable and disable features. To update account
        settings, users need to be an account owner or users need to be assigned an IAM
        policy with the Administrator role for the Cloud Shell account management service.

        :param str account_id: The account ID in which the account settings belong
               to.
        :param str rev: (optional) Unique revision number for the settings object.
        :param bool default_enable_new_features: (optional) You can choose which
               Cloud Shell features are available in the account and whether any new
               features are enabled as they become available. The feature settings apply
               only to the enabled Cloud Shell locations.
        :param bool default_enable_new_regions: (optional) Set whether Cloud Shell
               is enabled in a specific location for the account. The location determines
               where user and session data are stored. By default, users are routed to the
               nearest available location.
        :param bool enabled: (optional) When enabled, Cloud Shell is available to
               all users in the account.
        :param List[Feature] features: (optional) List of Cloud Shell features.
        :param List[RegionSetting] regions: (optional) List of Cloud Shell region
               settings.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `AccountSettings` object
        """

        if account_id is None:
            raise ValueError('account_id must be provided')
        if features is not None:
            features = [convert_model(x) for x in features]
        if regions is not None:
            regions = [convert_model(x) for x in regions]
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V1', operation_id='update_account_settings'
        )
        headers.update(sdk_headers)

        data = {
            '_rev': rev,
            'default_enable_new_features': default_enable_new_features,
            'default_enable_new_regions': default_enable_new_regions,
            'enabled': enabled,
            'features': features,
            'regions': regions,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_id']
        path_param_values = self.encode_path_vars(account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/api/v1/user/accounts/{account_id}/settings'.format(**path_param_dict)
        request = self.prepare_request(method='POST', url=url, headers=headers, data=data)

        response = self.send(request)
        return response


##############################################################################
# Models
##############################################################################


class AccountSettings:
    """
    Definition of Cloud Shell account settings.

    :attr str id: (optional) Unique id of the settings object.
    :attr str rev: (optional) Unique revision number for the settings object.
    :attr str account_id: (optional) The id of the account the settings belong to.
    :attr int created_at: (optional) Creation timestamp in Unix epoch time.
    :attr str created_by: (optional) IAM ID of creator.
    :attr bool default_enable_new_features: (optional) You can choose which Cloud
          Shell features are available in the account and whether any new features are
          enabled as they become available. The feature settings apply only to the enabled
          Cloud Shell locations.
    :attr bool default_enable_new_regions: (optional) Set whether Cloud Shell is
          enabled in a specific location for the account. The location determines where
          user and session data are stored. By default, users are routed to the nearest
          available location.
    :attr bool enabled: (optional) When enabled, Cloud Shell is available to all
          users in the account.
    :attr List[Feature] features: (optional) List of Cloud Shell features.
    :attr List[RegionSetting] regions: (optional) List of Cloud Shell region
          settings.
    :attr str type: (optional) Type of api response object.
    :attr int updated_at: (optional) Timestamp of last update in Unix epoch time.
    :attr str updated_by: (optional) IAM ID of last updater.
    """

    def __init__(
        self,
        *,
        id: str = None,
        rev: str = None,
        account_id: str = None,
        created_at: int = None,
        created_by: str = None,
        default_enable_new_features: bool = None,
        default_enable_new_regions: bool = None,
        enabled: bool = None,
        features: List['Feature'] = None,
        regions: List['RegionSetting'] = None,
        type: str = None,
        updated_at: int = None,
        updated_by: str = None
    ) -> None:
        """
        Initialize a AccountSettings object.

        :param str rev: (optional) Unique revision number for the settings object.
        :param bool default_enable_new_features: (optional) You can choose which
               Cloud Shell features are available in the account and whether any new
               features are enabled as they become available. The feature settings apply
               only to the enabled Cloud Shell locations.
        :param bool default_enable_new_regions: (optional) Set whether Cloud Shell
               is enabled in a specific location for the account. The location determines
               where user and session data are stored. By default, users are routed to the
               nearest available location.
        :param bool enabled: (optional) When enabled, Cloud Shell is available to
               all users in the account.
        :param List[Feature] features: (optional) List of Cloud Shell features.
        :param List[RegionSetting] regions: (optional) List of Cloud Shell region
               settings.
        """
        self.id = id
        self.rev = rev
        self.account_id = account_id
        self.created_at = created_at
        self.created_by = created_by
        self.default_enable_new_features = default_enable_new_features
        self.default_enable_new_regions = default_enable_new_regions
        self.enabled = enabled
        self.features = features
        self.regions = regions
        self.type = type
        self.updated_at = updated_at
        self.updated_by = updated_by

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'AccountSettings':
        """Initialize a AccountSettings object from a json dictionary."""
        args = {}
        if '_id' in _dict:
            args['id'] = _dict.get('_id')
        if '_rev' in _dict:
            args['rev'] = _dict.get('_rev')
        if 'account_id' in _dict:
            args['account_id'] = _dict.get('account_id')
        if 'created_at' in _dict:
            args['created_at'] = _dict.get('created_at')
        if 'created_by' in _dict:
            args['created_by'] = _dict.get('created_by')
        if 'default_enable_new_features' in _dict:
            args['default_enable_new_features'] = _dict.get('default_enable_new_features')
        if 'default_enable_new_regions' in _dict:
            args['default_enable_new_regions'] = _dict.get('default_enable_new_regions')
        if 'enabled' in _dict:
            args['enabled'] = _dict.get('enabled')
        if 'features' in _dict:
            args['features'] = [Feature.from_dict(x) for x in _dict.get('features')]
        if 'regions' in _dict:
            args['regions'] = [RegionSetting.from_dict(x) for x in _dict.get('regions')]
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        if 'updated_at' in _dict:
            args['updated_at'] = _dict.get('updated_at')
        if 'updated_by' in _dict:
            args['updated_by'] = _dict.get('updated_by')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a AccountSettings object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and getattr(self, 'id') is not None:
            _dict['_id'] = getattr(self, 'id')
        if hasattr(self, 'rev') and self.rev is not None:
            _dict['_rev'] = self.rev
        if hasattr(self, 'account_id') and getattr(self, 'account_id') is not None:
            _dict['account_id'] = getattr(self, 'account_id')
        if hasattr(self, 'created_at') and getattr(self, 'created_at') is not None:
            _dict['created_at'] = getattr(self, 'created_at')
        if hasattr(self, 'created_by') and getattr(self, 'created_by') is not None:
            _dict['created_by'] = getattr(self, 'created_by')
        if hasattr(self, 'default_enable_new_features') and self.default_enable_new_features is not None:
            _dict['default_enable_new_features'] = self.default_enable_new_features
        if hasattr(self, 'default_enable_new_regions') and self.default_enable_new_regions is not None:
            _dict['default_enable_new_regions'] = self.default_enable_new_regions
        if hasattr(self, 'enabled') and self.enabled is not None:
            _dict['enabled'] = self.enabled
        if hasattr(self, 'features') and self.features is not None:
            _dict['features'] = [x.to_dict() for x in self.features]
        if hasattr(self, 'regions') and self.regions is not None:
            _dict['regions'] = [x.to_dict() for x in self.regions]
        if hasattr(self, 'type') and getattr(self, 'type') is not None:
            _dict['type'] = getattr(self, 'type')
        if hasattr(self, 'updated_at') and getattr(self, 'updated_at') is not None:
            _dict['updated_at'] = getattr(self, 'updated_at')
        if hasattr(self, 'updated_by') and getattr(self, 'updated_by') is not None:
            _dict['updated_by'] = getattr(self, 'updated_by')
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this AccountSettings object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'AccountSettings') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'AccountSettings') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Feature:
    """
    Describes a Cloud Shell feature.

    :attr bool enabled: (optional) State of the feature.
    :attr str key: (optional) Name of the feature.
    """

    def __init__(self, *, enabled: bool = None, key: str = None) -> None:
        """
        Initialize a Feature object.

        :param bool enabled: (optional) State of the feature.
        :param str key: (optional) Name of the feature.
        """
        self.enabled = enabled
        self.key = key

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Feature':
        """Initialize a Feature object from a json dictionary."""
        args = {}
        if 'enabled' in _dict:
            args['enabled'] = _dict.get('enabled')
        if 'key' in _dict:
            args['key'] = _dict.get('key')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Feature object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'enabled') and self.enabled is not None:
            _dict['enabled'] = self.enabled
        if hasattr(self, 'key') and self.key is not None:
            _dict['key'] = self.key
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Feature object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'Feature') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Feature') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class RegionSetting:
    """
    Describes a Cloud Shell region setting.

    :attr bool enabled: (optional) State of the region.
    :attr str key: (optional) Name of the region.
    """

    def __init__(self, *, enabled: bool = None, key: str = None) -> None:
        """
        Initialize a RegionSetting object.

        :param bool enabled: (optional) State of the region.
        :param str key: (optional) Name of the region.
        """
        self.enabled = enabled
        self.key = key

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RegionSetting':
        """Initialize a RegionSetting object from a json dictionary."""
        args = {}
        if 'enabled' in _dict:
            args['enabled'] = _dict.get('enabled')
        if 'key' in _dict:
            args['key'] = _dict.get('key')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RegionSetting object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'enabled') and self.enabled is not None:
            _dict['enabled'] = self.enabled
        if hasattr(self, 'key') and self.key is not None:
            _dict['key'] = self.key
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RegionSetting object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RegionSetting') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RegionSetting') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2021.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# IBM OpenAPI SDK Code Generator Version: 3.41.0-f1ef0102-20211018-193503

"""
Manage lifecycle of your Cloud resource groups using Resource Manager APIs.

API Version: 2.0
"""

from datetime import datetime
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import datetime_to_string, string_to_datetime

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################


class ResourceManagerV2(BaseService):
    """The Resource Manager V2 service."""

    DEFAULT_SERVICE_URL = 'https://resource-controller.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'resource_manager'

    @classmethod
    def new_instance(
        cls,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> 'ResourceManagerV2':
        """
        Return a new client for the Resource Manager service using the specified
               parameters and external configuration.
        """
        authenticator = get_authenticator_from_environment(service_name)
        service = cls(authenticator)
        service.configure_service(service_name)
        return service

    def __init__(
        self,
        authenticator: Authenticator = None,
    ) -> None:
        """
        Construct a new client for the Resource Manager service.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/main/README.md
               about initializing the authenticator of your choice.
        """
        BaseService.__init__(self, service_url=self.DEFAULT_SERVICE_URL, authenticator=authenticator)

    #########################
    # Resource Group
    #########################

    def list_resource_groups(
        self,
        *,
        account_id: str = None,
        date: str = None,
        name: str = None,
        default: bool = None,
        include_deleted: bool = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Get a list of all resource groups.

        Call this method to retrieve information about all resource groups and associated
        quotas in an account. The `id` returned in the response can be used to [create a
        resource instance
        later](https://cloud.ibm.com/apidocs/resource-controller/resource-controller?code=java#create-resource-instance).
        The response can be filtered based on queryParams such as `account_id`, `name`,
        `default`, and more to narrow your search.Users need to be assigned IAM policies
        with the Viewer role or higher on the targeted resource groups.

        :param str account_id: (optional) The ID of the account that contains the
               resource groups that you want to get.
        :param str date: (optional) The date in the format of YYYY-MM which returns
               resource groups. Deleted resource groups will be excluded before this
               month.
        :param str name: (optional) The name of the resource group.
        :param bool default: (optional) Boolean value to specify whether or not to
               list default resource groups.
        :param bool include_deleted: (optional) Boolean value to specify whether or
               not to list default resource groups.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ResourceGroupList` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V2', operation_id='list_resource_groups'
        )
        headers.update(sdk_headers)

        params = {
            'account_id': account_id,
            'date': date,
            'name': name,
            'default': default,
            'include_deleted': include_deleted,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        url = '/v2/resource_groups'
        request = self.prepare_request(method='GET', url=url, headers=headers, params=params)

        response = self.send(request, **kwargs)
        return response

    def create_resource_group(self, *, name: str = None, account_id: str = None, **kwargs) -> DetailedResponse:
        """
        Create a resource group.

        Create a resource group in an account to organize your account resources in
        customizable groupings so that you can quickly assign users access to more than
        one resource at a time. To learn what makes a good resource group strategy, see
        [Best practices for organizing
        resources](https://cloud.ibm.com/docs/account?topic=account-account_setup). A
        default resource group is created when an account is created. If you have a Lite
        account or 30-day trial, you cannot create extra resource groups, but you can
        rename your default resource group. If you have a Pay-As-You-Go or Subscription
        account, you can create multiple resource groups. You must be assigned an IAM
        policy with the Administrator role on All Account Management services to create
        extra resource groups.

        :param str name: (optional) The new name of the resource group.
        :param str account_id: (optional) The account id of the resource group.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ResCreateResourceGroup` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V2', operation_id='create_resource_group'
        )
        headers.update(sdk_headers)

        data = {'name': name, 'account_id': account_id}
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        url = '/v2/resource_groups'
        request = self.prepare_request(method='POST', url=url, headers=headers, data=data)

        response = self.send(request, **kwargs)
        return response

    def get_resource_group(self, id: str, **kwargs) -> DetailedResponse:
        """
        Get a resource group.

        Retrieve a resource group by alias ID. Call this method to get details about a
        particular resource group, like the name of the resource group, associated quotas,
        whether the state is active, the resource group ID and the CRN. The `id` returned
        in the response can be used to [create a resource instance
        later](https://cloud.ibm.com/apidocs/resource-controller/resource-controller?code=java#create-resource-instance).
        Users need to be assigned an IAM policy with the Viewer role or higher on the
        targeted resource group.

        :param str id: The short or long ID of the alias.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ResourceGroup` object
        """

        if id is None:
            raise ValueError('id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V2', operation_id='get_resource_group'
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        path_param_keys = ['id']
        path_param_values = self.encode_path_vars(id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/resource_groups/{id}'.format(**path_param_dict)
        request = self.prepare_request(method='GET', url=url, headers=headers)

        response = self.send(request, **kwargs)
        return response

    def update_resource_group(self, id: str, *, name: str = None, state: str = None, **kwargs) -> DetailedResponse:
        """
        Update a resource group.

        Update a resource group by the alias ID. Call this method to update information
        about an existing resource group. You can rename a resource group and activate or
        suspend a particular resource group. To update a resource group, users need to be
        assigned with IAM policies with the Editor role or higher.

        :param str id: The short or long ID of the alias.
        :param str name: (optional) The new name of the resource group.
        :param str state: (optional) The state of the resource group.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ResourceGroup` object
        """

        if id is None:
            raise ValueError('id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V2', operation_id='update_resource_group'
        )
        headers.update(sdk_headers)

        data = {'name': name, 'state': state}
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        path_param_keys = ['id']
        path_param_values = self.encode_path_vars(id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/resource_groups/{id}'.format(**path_param_dict)
        request = self.prepare_request(method='PATCH', url=url, headers=headers, data=data)

        response = self.send(request, **kwargs)
        return response

    def delete_resource_group(self, id: str, **kwargs) -> DetailedResponse:
        """
        Delete a resource group.

        Delete a resource group by the alias ID. You can delete a resource group only if
        the targeted resource group does not contain any resources or if it is not a
        default resource group. When a user creates an account, a default resource group
        is created in the account. If you want to delete a resource group that contains
        resources, first [delete the resource
        instances](https://cloud.ibm.com/apidocs/resource-controller/resource-controller?code=java#delete-resource-instance).
        Then, delete the resource group when all resource instances in the group are
        deleted. Users need to be assigned an IAM policy with the Editor role or higher on
        the targeted resource group.

        :param str id: The short or long ID of the alias.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if id is None:
            raise ValueError('id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V2', operation_id='delete_resource_group'
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        path_param_keys = ['id']
        path_param_values = self.encode_path_vars(id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/resource_groups/{id}'.format(**path_param_dict)
        request = self.prepare_request(method='DELETE', url=url, headers=headers)

        response = self.send(request, **kwargs)
        return response

    #########################
    # Quota Definition
    #########################

    def list_quota_definitions(self, **kwargs) -> DetailedResponse:
        """
        List quota definitions.

        Get a list of all quota definitions. Quotas for a resource group limit the number
        of apps, instances, and memory allowed for that specific resource group. Each
        resource group that you have on your account has a specific set of quotas.
        Standard quotas are for resource groups that are created by users with a Lite
        account, and Pay-As-You-Go quotas are for resource groups that are created with a
        Pay-As-You-Go account. This method provides list of all available quota
        definitions. No specific IAM policy needed.

        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `QuotaDefinitionList` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V2', operation_id='list_quota_definitions'
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        url = '/v2/quota_definitions'
        request = self.prepare_request(method='GET', url=url, headers=headers)

        response = self.send(request, **kwargs)
        return response

    def get_quota_definition(self, id: str, **kwargs) -> DetailedResponse:
        """
        Get a quota definition.

        Call this method to retrieve information about a particular quota by passing the
        quota ID. The response can be used to identify the quota type, Standard or Paid.
        Information about available resources, such as number of apps, number of service
        instances, and memory, are returned in the response. Quotas for a resource group
        limit the number of apps, instances, and memory allowed for that specific resource
        group. Each resource group that you have on your account has a specific set of
        quotas. Standard quotas are for resource groups that are created by users with a
        Lite account, and Pay-As-You-Go quotas are for resource groups that are created
        with a Pay-As-You-Go account. No specific IAM policy needed.

        :param str id: The id of the quota.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `QuotaDefinition` object
        """

        if id is None:
            raise ValueError('id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME, service_version='V2', operation_id='get_quota_definition'
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        headers['Accept'] = 'application/json'

        path_param_keys = ['id']
        path_param_values = self.encode_path_vars(id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/v2/quota_definitions/{id}'.format(**path_param_dict)
        request = self.prepare_request(method='GET', url=url, headers=headers)

        response = self.send(request, **kwargs)
        return response


##############################################################################
# Models
##############################################################################


class QuotaDefinition:
    """
    A returned quota definition.

    :attr str id: (optional) An alpha-numeric value identifying the quota.
    :attr str name: (optional) The human-readable name of the quota.
    :attr str type: (optional) The type of the quota.
    :attr float number_of_apps: (optional) The total app limit.
    :attr float number_of_service_instances: (optional) The total service instances
          limit per app.
    :attr float default_number_of_instances_per_lite_plan: (optional) Default number
          of instances per lite plan.
    :attr float instances_per_app: (optional) The total instances limit per app.
    :attr str instance_memory: (optional) The total memory of app instance.
    :attr str total_app_memory: (optional) The total app memory capacity.
    :attr float vsi_limit: (optional) The VSI limit.
    :attr List[ResourceQuota] resource_quotas: (optional) The resource quotas
          associated with a quota definition.
    :attr datetime created_at: (optional) The date when the quota was initially
          created.
    :attr datetime updated_at: (optional) The date when the quota was last updated.
    """

    def __init__(
        self,
        *,
        id: str = None,
        name: str = None,
        type: str = None,
        number_of_apps: float = None,
        number_of_service_instances: float = None,
        default_number_of_instances_per_lite_plan: float = None,
        instances_per_app: float = None,
        instance_memory: str = None,
        total_app_memory: str = None,
        vsi_limit: float = None,
        resource_quotas: List['ResourceQuota'] = None,
        created_at: datetime = None,
        updated_at: datetime = None
    ) -> None:
        """
        Initialize a QuotaDefinition object.

        :param str id: (optional) An alpha-numeric value identifying the quota.
        :param str name: (optional) The human-readable name of the quota.
        :param str type: (optional) The type of the quota.
        :param float number_of_apps: (optional) The total app limit.
        :param float number_of_service_instances: (optional) The total service
               instances limit per app.
        :param float default_number_of_instances_per_lite_plan: (optional) Default
               number of instances per lite plan.
        :param float instances_per_app: (optional) The total instances limit per
               app.
        :param str instance_memory: (optional) The total memory of app instance.
        :param str total_app_memory: (optional) The total app memory capacity.
        :param float vsi_limit: (optional) The VSI limit.
        :param List[ResourceQuota] resource_quotas: (optional) The resource quotas
               associated with a quota definition.
        :param datetime created_at: (optional) The date when the quota was
               initially created.
        :param datetime updated_at: (optional) The date when the quota was last
               updated.
        """
        self.id = id
        self.name = name
        self.type = type
        self.number_of_apps = number_of_apps
        self.number_of_service_instances = number_of_service_instances
        self.default_number_of_instances_per_lite_plan = default_number_of_instances_per_lite_plan
        self.instances_per_app = instances_per_app
        self.instance_memory = instance_memory
        self.total_app_memory = total_app_memory
        self.vsi_limit = vsi_limit
        self.resource_quotas = resource_quotas
        self.created_at = created_at
        self.updated_at = updated_at

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'QuotaDefinition':
        """Initialize a QuotaDefinition object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        if 'number_of_apps' in _dict:
            args['number_of_apps'] = _dict.get('number_of_apps')
        if 'number_of_service_instances' in _dict:
            args['number_of_service_instances'] = _dict.get('number_of_service_instances')
        if 'default_number_of_instances_per_lite_plan' in _dict:
            args['default_number_of_instances_per_lite_plan'] = _dict.get('default_number_of_instances_per_lite_plan')
        if 'instances_per_app' in _dict:
            args['instances_per_app'] = _dict.get('instances_per_app')
        if 'instance_memory' in _dict:
            args['instance_memory'] = _dict.get('instance_memory')
        if 'total_app_memory' in _dict:
            args['total_app_memory'] = _dict.get('total_app_memory')
        if 'vsi_limit' in _dict:
            args['vsi_limit'] = _dict.get('vsi_limit')
        if 'resource_quotas' in _dict:
            args['resource_quotas'] = [ResourceQuota.from_dict(x) for x in _dict.get('resource_quotas')]
        if 'created_at' in _dict:
            args['created_at'] = string_to_datetime(_dict.get('created_at'))
        if 'updated_at' in _dict:
            args['updated_at'] = string_to_datetime(_dict.get('updated_at'))
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a QuotaDefinition object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'number_of_apps') and self.number_of_apps is not None:
            _dict['number_of_apps'] = self.number_of_apps
        if hasattr(self, 'number_of_service_instances') and self.number_of_service_instances is not None:
            _dict['number_of_service_instances'] = self.number_of_service_instances
        if (
            hasattr(self, 'default_number_of_instances_per_lite_plan')
            and self.default_number_of_instances_per_lite_plan is not None
        ):
            _dict['default_number_of_instances_per_lite_plan'] = self.default_number_of_instances_per_lite_plan
        if hasattr(self, 'instances_per_app') and self.instances_per_app is not None:
            _dict['instances_per_app'] = self.instances_per_app
        if hasattr(self, 'instance_memory') and self.instance_memory is not None:
            _dict['instance_memory'] = self.instance_memory
        if hasattr(self, 'total_app_memory') and self.total_app_memory is not None:
            _dict['total_app_memory'] = self.total_app_memory
        if hasattr(self, 'vsi_limit') and self.vsi_limit is not None:
            _dict['vsi_limit'] = self.vsi_limit
        if hasattr(self, 'resource_quotas') and self.resource_quotas is not None:
            _dict['resource_quotas'] = [x.to_dict() for x in self.resource_quotas]
        if hasattr(self, 'created_at') and self.created_at is not None:
            _dict['created_at'] = datetime_to_string(self.created_at)
        if hasattr(self, 'updated_at') and self.updated_at is not None:
            _dict['updated_at'] = datetime_to_string(self.updated_at)
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this QuotaDefinition object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'QuotaDefinition') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'QuotaDefinition') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class QuotaDefinitionList:
    """
    A list of quota definitions.

    :attr List[QuotaDefinition] resources: The list of quota definitions.
    """

    def __init__(self, resources: List['QuotaDefinition']) -> None:
        """
        Initialize a QuotaDefinitionList object.

        :param List[QuotaDefinition] resources: The list of quota definitions.
        """
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'QuotaDefinitionList':
        """Initialize a QuotaDefinitionList object from a json dictionary."""
        args = {}
        if 'resources' in _dict:
            args['resources'] = [QuotaDefinition.from_dict(x) for x in _dict.get('resources')]
        else:
            raise ValueError('Required property \'resources\' not present in QuotaDefinitionList JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a QuotaDefinitionList object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'resources') and self.resources is not None:
            _dict['resources'] = [x.to_dict() for x in self.resources]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this QuotaDefinitionList object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'QuotaDefinitionList') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'QuotaDefinitionList') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ResCreateResourceGroup:
    """
    A newly-created resource group.

    :attr str id: (optional) An alpha-numeric value identifying the resource group.
    :attr str crn: (optional) The full CRN (cloud resource name) associated with the
          resource group. For more on this format, see [Cloud Resource
          Names](https://cloud.ibm.com/docs/account?topic=account-crn).
    """

    def __init__(self, *, id: str = None, crn: str = None) -> None:
        """
        Initialize a ResCreateResourceGroup object.

        :param str id: (optional) An alpha-numeric value identifying the resource
               group.
        :param str crn: (optional) The full CRN (cloud resource name) associated
               with the resource group. For more on this format, see [Cloud Resource
               Names](https://cloud.ibm.com/docs/account?topic=account-crn).
        """
        self.id = id
        self.crn = crn

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResCreateResourceGroup':
        """Initialize a ResCreateResourceGroup object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'crn' in _dict:
            args['crn'] = _dict.get('crn')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResCreateResourceGroup object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'crn') and self.crn is not None:
            _dict['crn'] = self.crn
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResCreateResourceGroup object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResCreateResourceGroup') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResCreateResourceGroup') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ResourceGroup:
    """
    A resource group.

    :attr str id: (optional) An alpha-numeric value identifying the resource group.
    :attr str crn: (optional) The full CRN (cloud resource name) associated with the
          resource group. For more on this format, see [Cloud Resource
          Names](https://cloud.ibm.com/docs/account?topic=account-crn).
    :attr str account_id: (optional) An alpha-numeric value identifying the account
          ID.
    :attr str name: (optional) The human-readable name of the resource group.
    :attr str state: (optional) The state of the resource group.
    :attr bool default: (optional) Identify if this resource group is default of the
          account or not.
    :attr str quota_id: (optional) An alpha-numeric value identifying the quota ID
          associated with the resource group.
    :attr str quota_url: (optional) The URL to access the quota details that
          associated with the resource group.
    :attr str payment_methods_url: (optional) The URL to access the payment methods
          details that associated with the resource group.
    :attr List[object] resource_linkages: (optional) An array of the resources that
          linked to the resource group.
    :attr str teams_url: (optional) The URL to access the team details that
          associated with the resource group.
    :attr datetime created_at: (optional) The date when the resource group was
          initially created.
    :attr datetime updated_at: (optional) The date when the resource group was last
          updated.
    """

    def __init__(
        self,
        *,
        id: str = None,
        crn: str = None,
        account_id: str = None,
        name: str = None,
        state: str = None,
        default: bool = None,
        quota_id: str = None,
        quota_url: str = None,
        payment_methods_url: str = None,
        resource_linkages: List[object] = None,
        teams_url: str = None,
        created_at: datetime = None,
        updated_at: datetime = None
    ) -> None:
        """
        Initialize a ResourceGroup object.

        :param str id: (optional) An alpha-numeric value identifying the resource
               group.
        :param str crn: (optional) The full CRN (cloud resource name) associated
               with the resource group. For more on this format, see [Cloud Resource
               Names](https://cloud.ibm.com/docs/account?topic=account-crn).
        :param str account_id: (optional) An alpha-numeric value identifying the
               account ID.
        :param str name: (optional) The human-readable name of the resource group.
        :param str state: (optional) The state of the resource group.
        :param bool default: (optional) Identify if this resource group is default
               of the account or not.
        :param str quota_id: (optional) An alpha-numeric value identifying the
               quota ID associated with the resource group.
        :param str quota_url: (optional) The URL to access the quota details that
               associated with the resource group.
        :param str payment_methods_url: (optional) The URL to access the payment
               methods details that associated with the resource group.
        :param List[object] resource_linkages: (optional) An array of the resources
               that linked to the resource group.
        :param str teams_url: (optional) The URL to access the team details that
               associated with the resource group.
        :param datetime created_at: (optional) The date when the resource group was
               initially created.
        :param datetime updated_at: (optional) The date when the resource group was
               last updated.
        """
        self.id = id
        self.crn = crn
        self.account_id = account_id
        self.name = name
        self.state = state
        self.default = default
        self.quota_id = quota_id
        self.quota_url = quota_url
        self.payment_methods_url = payment_methods_url
        self.resource_linkages = resource_linkages
        self.teams_url = teams_url
        self.created_at = created_at
        self.updated_at = updated_at

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResourceGroup':
        """Initialize a ResourceGroup object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'crn' in _dict:
            args['crn'] = _dict.get('crn')
        if 'account_id' in _dict:
            args['account_id'] = _dict.get('account_id')
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        if 'state' in _dict:
            args['state'] = _dict.get('state')
        if 'default' in _dict:
            args['default'] = _dict.get('default')
        if 'quota_id' in _dict:
            args['quota_id'] = _dict.get('quota_id')
        if 'quota_url' in _dict:
            args['quota_url'] = _dict.get('quota_url')
        if 'payment_methods_url' in _dict:
            args['payment_methods_url'] = _dict.get('payment_methods_url')
        if 'resource_linkages' in _dict:
            args['resource_linkages'] = _dict.get('resource_linkages')
        if 'teams_url' in _dict:
            args['teams_url'] = _dict.get('teams_url')
        if 'created_at' in _dict:
            args['created_at'] = string_to_datetime(_dict.get('created_at'))
        if 'updated_at' in _dict:
            args['updated_at'] = string_to_datetime(_dict.get('updated_at'))
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResourceGroup object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'crn') and self.crn is not None:
            _dict['crn'] = self.crn
        if hasattr(self, 'account_id') and self.account_id is not None:
            _dict['account_id'] = self.account_id
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        if hasattr(self, 'default') and self.default is not None:
            _dict['default'] = self.default
        if hasattr(self, 'quota_id') and self.quota_id is not None:
            _dict['quota_id'] = self.quota_id
        if hasattr(self, 'quota_url') and self.quota_url is not None:
            _dict['quota_url'] = self.quota_url
        if hasattr(self, 'payment_methods_url') and self.payment_methods_url is not None:
            _dict['payment_methods_url'] = self.payment_methods_url
        if hasattr(self, 'resource_linkages') and self.resource_linkages is not None:
            _dict['resource_linkages'] = self.resource_linkages
        if hasattr(self, 'teams_url') and self.teams_url is not None:
            _dict['teams_url'] = self.teams_url
        if hasattr(self, 'created_at') and self.created_at is not None:
            _dict['created_at'] = datetime_to_string(self.created_at)
        if hasattr(self, 'updated_at') and self.updated_at is not None:
            _dict['updated_at'] = datetime_to_string(self.updated_at)
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResourceGroup object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResourceGroup') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResourceGroup') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ResourceGroupList:
    """
    A list of resource groups.

    :attr List[ResourceGroup] resources: The list of resource groups.
    """

    def __init__(self, resources: List['ResourceGroup']) -> None:
        """
        Initialize a ResourceGroupList object.

        :param List[ResourceGroup] resources: The list of resource groups.
        """
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResourceGroupList':
        """Initialize a ResourceGroupList object from a json dictionary."""
        args = {}
        if 'resources' in _dict:
            args['resources'] = [ResourceGroup.from_dict(x) for x in _dict.get('resources')]
        else:
            raise ValueError('Required property \'resources\' not present in ResourceGroupList JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResourceGroupList object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'resources') and self.resources is not None:
            _dict['resources'] = [x.to_dict() for x in self.resources]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResourceGroupList object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResourceGroupList') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResourceGroupList') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ResourceQuota:
    """
    A resource quota.

    :attr str id: (optional) An alpha-numeric value identifying the quota.
    :attr str resource_id: (optional) The human-readable name of the quota.
    :attr str crn: (optional) The full CRN (cloud resource name) associated with the
          quota. For more on this format, see
          https://cloud.ibm.com/docs/account?topic=account-crn.
    :attr float limit: (optional) The limit number of this resource.
    """

    def __init__(self, *, id: str = None, resource_id: str = None, crn: str = None, limit: float = None) -> None:
        """
        Initialize a ResourceQuota object.

        :param str id: (optional) An alpha-numeric value identifying the quota.
        :param str resource_id: (optional) The human-readable name of the quota.
        :param str crn: (optional) The full CRN (cloud resource name) associated
               with the quota. For more on this format, see
               https://cloud.ibm.com/docs/account?topic=account-crn.
        :param float limit: (optional) The limit number of this resource.
        """
        self.id = id
        self.resource_id = resource_id
        self.crn = crn
        self.limit = limit

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResourceQuota':
        """Initialize a ResourceQuota object from a json dictionary."""
        args = {}
        if '_id' in _dict:
            args['id'] = _dict.get('_id')
        if 'resource_id' in _dict:
            args['resource_id'] = _dict.get('resource_id')
        if 'crn' in _dict:
            args['crn'] = _dict.get('crn')
        if 'limit' in _dict:
            args['limit'] = _dict.get('limit')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResourceQuota object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['_id'] = self.id
        if hasattr(self, 'resource_id') and self.resource_id is not None:
            _dict['resource_id'] = self.resource_id
        if hasattr(self, 'crn') and self.crn is not None:
            _dict['crn'] = self.crn
        if hasattr(self, 'limit') and self.limit is not None:
            _dict['limit'] = self.limit
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResourceQuota object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResourceQuota') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResourceQuota') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other
